/*
  mergemod.h
  
  This file is part of mergemem by Philipp Richter & Philipp Reisner

  mergemem is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  mergemem is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with mergemem; see the file COPYING.  If not, write to
  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <linux/ioctl.h>

/* return values issued by mergemem() */

#define MERGEMEM_SUCCESS 0
#define MERGEMEM_NOTASK1 1
#define MERGEMEM_NOTASK2 2
#define MERGEMEM_NOPAGE1 3
#define MERGEMEM_NOPAGE2 4
#define MERGEMEM_NOTEQUAL 5
#define MERGEMEM_ALREADYSH 6
#define MERGEMEM_MOREMAP 7
#define MERGEMEM_NOANNO 8
#define MERGEMEM_PLOCKED 9

/* if pid2==0 a checksum of pid1's page at the virtual address addr1 is
 * calculated and returned in addr2!!
 * In pid2 the number of mappings is returned.
 * On success MERGEMEM_SUCCESS is returned by write
 */
/* if pid1!=0 && pid2!=0 and the return value is MERGEMEM_SUCCESS the
 * two pages where merged.
 */


/* minor of the mergemem device */
#define MERGEMEM_MINOR 156
/* module name as printed in printk msgs */
#define MODULE_NAME "mergemod: "

/* ioctl structures */

struct mergemem_mmem
{
  pid_t pid1;
  unsigned long addr1;
  pid_t pid2;
  unsigned long addr2;
};

struct mergemem_chksum
{
  pid_t pid;                  /* in  - pid */
  unsigned long addr;         /* in  - addr */
  unsigned long chksum;       /* out - checksum */
  int nrefs;                  /* out - references count of page */
};

/* ioctl commands */
#define MERGEMEM_CHECK_VER     _IOR( 'M', 0x00, int )
#define MERGEMEM_MERGE_MEM     _IOW( 'M', 0x01, struct mergemem_mmem )
#define MERGEMEM_GEN_CHECKSUM  _IOWR( 'M', 0x02, struct mergemem_chksum )
#define MERGEMEM_RESET_STAT    _IO(  'M', 0x03 )

